// ------------------------------------------------------------------------------------------------------
// LightningChart® example code: Demo shows Histogram plot using bars. 
//
// If you need any assistance, or notice error in this example code, please contact support@lightningchart.com. 
//
// Permission to use this code in your application comes with LightningChart® license. 
//
// https://www.arction.com | support@lightningchart.com | sales@lightningchart.com
//
// © Arction Ltd 2009-2021. All rights reserved.  
// ------------------------------------------------------------------------------------------------------
using Arction.Wpf.Charting;
using Arction.Wpf.Charting.Axes;
using Arction.Wpf.Charting.SeriesXY;
using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Threading;

namespace InteractiveExamples
{
    /// <summary>
    /// Interaction logic for ExampleHistogramXY.xaml
    /// </summary>
    public partial class ExampleHistogramXY : Window, IDisposable
    {
        private LightningChart _chart;

        private RowDataGenerator _rowDataGenerator;

        public ExampleHistogramXY()
        {
            _rowDataGenerator = new RowDataGenerator
            {
                IntervalMs = 10
            };
            _rowDataGenerator.NewDataGenerated += _rowDataGenerator_NewDataGenerated;
            _rowDataGenerator.Started += _rowDataGenerator_Started;
            _rowDataGenerator.Stopped += _rowDataGenerator_Stopped;
            _rowDataGenerator.Variation = 1;
            _rowDataGenerator.RowLength = 51;
            _rowDataGenerator.MaxValue = 500;
            _rowDataGenerator.MinValue = 0;
            _rowDataGenerator.InitialValue = 200;
            InitializeComponent();
            CreateChart();

            Start();

        }

        internal bool IsRunning
        {
            get
            {
                return _rowDataGenerator.IsRunning;
            }
        }

        public void Start()
        {
            if (IsRunning == false)
            {
                _rowDataGenerator.Start();
            }
        }

        public void Stop()
        {
            if (IsRunning == false)
            {
                _rowDataGenerator.Stop();
            }
        }
        private void CreateChart()
        {
            _chart = new LightningChart();

            _chart.BeginUpdate();

            _chart.ChartName = "Histogram Chart";

            _chart.ViewXY.DropOldSeriesData = true;

            _chart.ViewXY.BarSeries.Clear();

            _chart.ViewXY.XAxes[0].Visible = true;
            _chart.ViewXY.XAxes[0].MajorDiv = 0.0000001;
            _chart.ViewXY.XAxes[0].MinorDivCount = 0;
            _chart.ViewXY.XAxes[0].AutoDivSpacing = false;
            _chart.ViewXY.XAxes[0].ScrollMode = XAxisScrollMode.None;

            //Get y-axis and initialize it
            AxisY yAxis = _chart.ViewXY.YAxes[0];
            yAxis.Maximum = 500;
            yAxis.Minimum = 0;
            yAxis.AutoDivSeparationPercent = 200;
            yAxis.Title.DistanceToAxis = 50;
            yAxis.Title.Text = "Power spectrum P(f)";

            //Create bar series using the y-axis
            BarSeries bs1 = new BarSeries(_chart.ViewXY, _chart.ViewXY.XAxes[0], yAxis);
            bs1.Title.HorizontalAlign = AlignmentHorizontal.Right;
            bs1.Title.Color = Colors.Magenta;
            bs1.Title.Visible = false;
            bs1.Fill.Color = Colors.Orange;
            bs1.Fill.GradientFill = GradientFill.Solid;
            bs1.Fill.Style = RectFillStyle.ColorOnly;
            bs1.BarThickness = 20;
            bs1.Shadow.Visible = false;
            bs1.Shadow.Offset.SetValues(2, 2);
            bs1.BorderColor = ChartTools.CalcGradient(bs1.Fill.Color, Colors.Black, 70);
            bs1.LabelStyle.Visible = true;
            bs1.AllowUserInteraction = false;
            _chart.ViewXY.BarSeries.Add(bs1);

            //Setup bar view options
            _chart.ViewXY.BarViewOptions.BarSpacing = 1;
            _chart.ViewXY.BarViewOptions.Grouping = BarsGrouping.ByLocation;
            _chart.ViewXY.BarViewOptions.Stacking = BarsStacking.None;
            _chart.ViewXY.BarViewOptions.IndexGroupingFitGroupDistance = 0;
            _chart.ViewXY.BarViewOptions.IndexGroupingFitSideMargins = 10;

            //Don't show legend box
            _chart.ViewXY.LegendBoxes[0].Visible = false;


            gridChart.Children.Add(_chart);

            //Allow chart rendering
            _chart.EndUpdate();

        }
        private void _rowDataGenerator_Started(object sender, EventArgs e)
        {
            _rowDataGenerator.RowLength = int.Parse(textBoxResolution.Text);
        }

        private void _rowDataGenerator_Stopped(object sender, EventArgs e)
        {
        }
        private void _rowDataGenerator_NewDataGenerated(NewDataGeneratedEventArgs args)
        {
            if (_chart == null)
            {
                return;
            }

            BarSeries bs = _chart.ViewXY.BarSeries[0];
            int valueCount = args.Data.Length;
            BarSeriesValue[] values = new BarSeriesValue[valueCount];

            for (int i = 0; i < valueCount; i++)
            {
                values[i].Location = i;
                values[i].Value = args.Data[i];
                values[i].Text = i.ToString();
            }

            bs.Values = values;
        }
        public void Dispose()
        {
            Stop();

            // Don't forget to clear chart grid child list.
            gridChart.Children.Clear();

            if (_chart != null)
            {
                _chart.Dispose();
                _chart = null;
            }


        }
        public class RowDataGenerator
        {
            public delegate void NewDataGeneratedEventHandler(NewDataGeneratedEventArgs args);

            public event NewDataGeneratedEventHandler NewDataGenerated;
            public event EventHandler Started;
            public event EventHandler Stopped;

            private double[] m_aDataRow;
            private bool m_bRememberPrevData;
            private DispatcherTimer _dispatcherTimer;
            private double m_dInitialValue;
            private double m_dMax;
            private double m_dMin;
            private double m_dVariation;
            private int m_iRowLength;
            private double m_dPeak1X = 0;//peak 1 center 
            private double m_dPeak2X = 0;//peak 2 center 
            private Random _rand = new Random((int)(DateTime.Now.Ticks % int.MaxValue));

            public RowDataGenerator()
            {
                m_aDataRow = null;
                m_bRememberPrevData = true;
                m_dInitialValue = 10.0;
                m_dMax = 100.0;
                m_dMin = 0.0;
                m_dVariation = 0.0;
                m_iRowLength = 10;

                _dispatcherTimer = new DispatcherTimer();
                _dispatcherTimer.Tick += new EventHandler(m_dispatcherTimer_Tick);


            }

            public bool IsRunning
            {
                get
                {
                    return _dispatcherTimer.IsEnabled;
                }
            }

            /// <summary>
            /// Generator output interval in milliseconds.
            /// </summary>
            public int IntervalMs
            {
                get
                {
                    return _dispatcherTimer.Interval.Milliseconds;
                }

                set
                {
                    _dispatcherTimer.Interval = new TimeSpan(0, 0, 0, 0, value);
                    _interval = value;

                }
            }

            /// <summary>
            /// Remember previous row data. New row data is calculated related to it.
            /// </summary>
            public bool RememberPreviousData
            {
                get
                {
                    return m_bRememberPrevData;
                }

                set
                {
                    m_bRememberPrevData = value;
                }
            }

            /// <summary>
            /// Variation between adjacent points in range 0...100.
            /// </summary>
            public double Variation
            {
                get
                {
                    return m_dVariation;
                }

                set
                {
                    if (value < 0.0)
                    {
                        value = 0.0;
                    }
                    else if (value > 100.0)
                    {
                        value = 100.0;
                    }

                    m_dVariation = value;

                }
            }

            /// <summary>
            /// Frequency stability in range 0...100. Frequency drifting also depends on Variation, which effects especially in low Frequency Stability amounts. 
            /// </summary>
            public double FrequencyStability
            {
                get
                {
                    return m_dFrequencyStability;
                }
                set
                {
                    if (value < 1)
                    {
                        value = 1;
                    }

                    if (value > 100)
                    {
                        value = 100;
                    }

                    m_dFrequencyStability = value;
                }
            }

            private double m_dFrequencyStability = 1;
            private int _interval;

            /// <summary>
            /// Output value range minimum.
            /// </summary>
            public double MinValue
            {
                get
                {
                    return m_dMin;
                }

                set
                {
                    if (value > m_dMax)
                    {
                        value = m_dMax - 0.1;
                    }

                    m_dMin = value;
                }
            }

            /// <summary>
            /// Output value range maximum.
            /// </summary>
            public double MaxValue
            {
                get
                {
                    return m_dMax;
                }

                set
                {
                    if (value <= m_dMin)
                    {
                        value = m_dMin + 0.1;
                    }

                    m_dMax = value;
                }
            }

            /// <summary>
            /// Created data array length.
            /// </summary>
            public int RowLength
            {
                get
                {
                    return m_iRowLength;
                }

                set
                {
                    if (value < 1)
                    {
                        value = 1;
                    }

                    m_iRowLength = value;
                    m_aDataRow = null;

                    m_dPeak1X = m_iRowLength / 8.0;
                    m_dPeak2X = m_iRowLength / 2.0;
                }
            }

            /// <summary>
            /// Initial value of the first row.
            /// </summary>
            public double InitialValue
            {
                get
                {
                    return m_dInitialValue;
                }

                set
                {
                    m_dInitialValue = value;
                }
            }

            public bool IsEnabled { get; internal set; }

            /// <summary>
            /// Start the generator.
            /// </summary>
            public void Start()
            {
                if (_dispatcherTimer.IsEnabled == false)
                {
                    int iIntervalInMs = 0;

                    m_dPeak1X = m_iRowLength / 8.0;
                    m_dPeak2X = m_iRowLength / 2.0;

                    iIntervalInMs = _interval;
                    _dispatcherTimer.Interval = new TimeSpan(0, 0, 0, 0, iIntervalInMs);
                    _dispatcherTimer.Start();

                    if (Started != null)
                    {
                        Started(this, null);
                    }
                }
            }

            /// <summary>
            /// Stop the generator.
            /// </summary>
            public void Stop()
            {
                if (_dispatcherTimer.IsEnabled == true)
                {
                    _dispatcherTimer.Stop();

                    if (Stopped != null)
                    {
                        Stopped(this, null);
                    }
                }
            }


            /// <summary>
            /// Generate data.
            /// </summary>
            public double[] GenerateData()
            {
                double[] aNewData = new double[m_iRowLength];

                for (int i = 0; i < m_iRowLength; i++)
                {
                    aNewData[i] = m_dInitialValue;
                }

                if (m_bRememberPrevData && m_aDataRow != null)
                {
                    //Copy old data as basis 
                    int iCopyCount = m_aDataRow.Length;
                    if (m_iRowLength < iCopyCount)
                    {
                        iCopyCount = m_iRowLength;
                    }

                    for (int i = 0; i < m_iRowLength; i++)
                    {
                        //aNewData[i] = m_aDataRow[i];
                        aNewData[i] = 10;
                    }
                }

                //Calc new data
                double dAmplitude = m_dVariation / 100.0 * (m_dMax - m_dMin);
                Random rand = _rand;

                double dHalf = m_iRowLength / 2.0;
                m_dPeak1X += (rand.NextDouble() - 0.5) * m_dVariation / m_dFrequencyStability / 100.0 * m_iRowLength / 2.0;
                m_dPeak2X += (rand.NextDouble() - 0.5) * m_dVariation / m_dFrequencyStability / 100.0 * m_iRowLength;
                if (m_dPeak1X < 0)
                {
                    m_dPeak1X = 0;
                }

                if (m_dPeak1X > m_iRowLength)
                {
                    m_dPeak1X = m_iRowLength;
                }

                if (m_dPeak2X < 0)
                {
                    m_dPeak2X = 0;
                }

                if (m_dPeak2X > m_iRowLength)
                {
                    m_dPeak2X = m_iRowLength;
                }

                double dNewValue1, dNewValue2, dX1, dX2;
                double dPeakY1 = m_dMax / 3.0 * 2.0;
                double dPeakY2 = m_dMax / 2.0;
                double dNarrowFactor1 = 8.0;
                double dNarrowFactor2 = 24.0;

                double dA1 = dPeakY1 / (dHalf * dHalf) * dNarrowFactor1;
                double dA2 = dPeakY2 / (dHalf * dHalf) * dNarrowFactor2;
                double dSum12;
                for (int i = 0; i < m_iRowLength; i++)
                {
                    dX1 = i - m_dPeak1X;
                    dX2 = i - m_dPeak2X;
                    dNewValue1 = dPeakY1 - dX1 * dX1 * dA1;
                    if (dNewValue1 < m_dMin)
                    {
                        dNewValue1 = m_dMin;
                    }

                    if (dNewValue1 > m_dMax)
                    {
                        dNewValue1 = m_dMax;
                    }

                    dNewValue2 = dPeakY2 - dX2 * dX2 * dA2;
                    if (dNewValue2 < m_dMin)
                    {
                        dNewValue2 = m_dMin;
                    }

                    if (dNewValue2 > m_dMax)
                    {
                        dNewValue2 = m_dMax;
                    }

                    dSum12 = dNewValue1 + dNewValue2;
                    dSum12 = dSum12 + dSum12 * (rand.NextDouble() - 0.5) * m_dVariation / 10.0;

                    aNewData[i] = (aNewData[i] + dSum12) / 2.0;
                    if (aNewData[i] < m_dMin)
                    {
                        aNewData[i] = m_dMin;
                    }

                    if (aNewData[i] > m_dMax)
                    {
                        aNewData[i] = m_dMax;
                    }
                }


                if (NewDataGenerated != null)
                {
                    NewDataGenerated(new NewDataGeneratedEventArgs(aNewData));
                }

                if (m_aDataRow == null || m_aDataRow.Length != m_iRowLength)
                {
                    m_aDataRow = new double[m_iRowLength];
                }

                Array.Copy(aNewData, m_aDataRow, aNewData.Length);

                return aNewData;
            }

            private void m_dispatcherTimer_Tick(object sender, EventArgs e)
            {
                GenerateData();
            }


            public void StartButton()
            {
                Start();
            }

            public void StopButton()
            {
                Stop();
            }

            public void VariationTextChanged(int iVariation)
            {
                m_dVariation = iVariation;
            }

            public void IntervalTextChanged(int iIntervalInMs)
            {
                _dispatcherTimer.Interval = new TimeSpan(0, 0, 0, 0, iIntervalInMs);
            }
        }

        public class NewDataGeneratedEventArgs
        {
            public double[] Data;

            public NewDataGeneratedEventArgs(double[] data)
            {
                Data = data;
            }

        }

        private void textBoxResolution_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (_chart != null)
            {
                int resolution = 51;
                int.TryParse(textBoxResolution.Text, out resolution);
                
                _rowDataGenerator.RowLength = resolution;

               
            }
        }

        private void buttonStart_Click(object sender, RoutedEventArgs e)
        {
            _rowDataGenerator.StartButton();
        }

        private void buttonStop_Click(object sender, RoutedEventArgs e)
        {
            _rowDataGenerator.StopButton();
        }

        private void textBoxExVariation_TextChanged(object sender, TextChangedEventArgs e)
        {
            int iVariation;
            if (int.TryParse(textBoxExVariation.Text, out iVariation))
            {
                _rowDataGenerator.VariationTextChanged(iVariation);
            }
        }

        private void textBoxExInterval_TextChanged(object sender, TextChangedEventArgs e)
        {
            int iInterval;
            if (int.TryParse(textBoxExInterval.Text, out iInterval))
            {
                _rowDataGenerator.IntervalTextChanged(iInterval);
            }
        }

        private void bt_get_bar_series_range_Click(object sender, RoutedEventArgs e)
        {
            if(_chart != null)
            {
                _chart.BeginUpdate();
                RectangleXY[] rect = _chart.ViewXY.BarSeries[0].GetBarRectangles();
                Console.WriteLine("=======================");
                double coord_x_screen_axies= 0;
                List<double> screen_postion = new List<double>();
                for (int count = 0; count < rect.Length; count++)
                {
                    _chart.ViewXY.XAxes[0].CoordToValue(rect[count].X, out coord_x_screen_axies, true);
                    screen_postion.Add(coord_x_screen_axies);
                    Console.WriteLine("count :" + count + "\t" + rect[count].X + "\t" + rect[count].Y +"\t"+ rect[count].Width+"\t"+  coord_x_screen_axies);

                }
                Console.WriteLine("=======================");
                //_chart.ViewXY.XAxes[0].SetRange(0, rect[rect.Length-1].X);
                //_chart.ViewXY.XAxes[0].Maximum = rect[rect.Length - 1].X;
                //_chart.ViewXY.XAxes[0].Minimum = 0;
                //ChartTools.ZoomToDataAndLabelsArea(_chart);
               
                Console.Write("_chart.ViewXY.XAxes[0].Minimum :" + _chart.ViewXY.XAxes[0].Minimum + "\t");
                Console.WriteLine("_chart.ViewXY.XAxes[0].Maximum :" + _chart.ViewXY.XAxes[0].Maximum);
                _chart.EndUpdate();
                InitializeCanvas(screen_postion);
            }
        }

        private void InitializeCanvas(List<double> screen_position)
        {

            info_grid.Width = gridChart.ActualWidth;
            Canvas temp = new Canvas();
            for(int count_position =0; count_position < screen_position.Count; count_position++)
            {
                TextBlock temp_textblock   = new TextBlock();
                temp_textblock.HorizontalAlignment = HorizontalAlignment.Stretch;
                temp_textblock.VerticalAlignment = VerticalAlignment.Stretch;
                temp_textblock.TextAlignment = TextAlignment.Center;
                temp_textblock.Text = (count_position + 1).ToString();
                Canvas.SetLeft(temp_textblock, screen_position[count_position]);
                Canvas.SetTop(temp_textblock, 10);
                temp.Children.Add(temp_textblock);
            }
            info_grid.Children.Add(temp);
        }
    }
}
